
-------------------------------------------------------------------------------
-- Optimization
-------------------------------------------------------------------------------

-- GLOBALS: LibStub, StaticPopup_Show

local DISABLED_FONT_COLOR = DISABLED_FONT_COLOR
local format = format
local HIGHLIGHT_FONT_COLOR = HIGHLIGHT_FONT_COLOR
local IsAddOnLoaded = IsAddOnLoaded
local next = next
local NORMAL_FONT_COLOR = NORMAL_FONT_COLOR
local rawset = rawset
local select = select
local setmetatable = setmetatable
local string = string

-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local A, B, C, D = unpack(SixUI) -- AddOn, Border, Constants, Database
local M = A:NewModule("Core Config")
local AceConfig = LibStub("AceConfig-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local AceDBOptions = LibStub("AceDBOptions-3.0")

-------------------------------------------------------------------------------
-- Module
-------------------------------------------------------------------------------

function M:OnInitialize()
	self:RegisterOptionsTables()
end

function M:RegisterOptionsTables()
	AceConfig:RegisterOptionsTable("SixUI", self.optionsTable, nil)
	self.optionsFrame = AceConfigDialog:AddToBlizOptions("SixUI", "SixUI")

	AceConfig:RegisterOptionsTable("SixUI General", self.GenerateGeneralOptionsTable, nil)
	self.generalOptionsFrame = AceConfigDialog:AddToBlizOptions("SixUI General", "General", "SixUI")

	AceConfig:RegisterOptionsTable("SixUI Action Bars", self.GenerateActionBarsOptionsTable, nil)
	self.actionBarsOptionFrame = AceConfigDialog:AddToBlizOptions("SixUI Action Bars", "Action Bars", "SixUI")

	AceConfig:RegisterOptionsTable("SixUI Displays", self.GenerateDisplaysOptionsTable, nil)
	self.displaysOptionsFrame = AceConfigDialog:AddToBlizOptions("SixUI Displays", "Displays", "SixUI")

	AceConfig:RegisterOptionsTable("SixUI Miscellaneous", self.GenerateMiscellaneousOptionsTable, nil)
	self.miscellaneousOptionsFrame = AceConfigDialog:AddToBlizOptions("SixUI Miscellaneous", "Miscellaneous", "SixUI")

	AceConfig:RegisterOptionsTable("SixUI Raid Frames", self.GenerateRaidFramesOptionsTable, nil)
	self.raidFramesOptionsFrame = AceConfigDialog:AddToBlizOptions("SixUI Raid Frames", "Raid Frames", "SixUI")

	AceConfig:RegisterOptionsTable("SixUI Profiles", AceDBOptions:GetOptionsTable(D.db), nil)
	self.profilesFrame = AceConfigDialog:AddToBlizOptions("SixUI Profiles", "Profiles", "SixUI")
end

-------------------------------------------------------------------------------
-- Options Table Helpers
-------------------------------------------------------------------------------

M.AnchorPointValues = {
	TOPLEFT = "Top Left",
	TOP = "Top",
	TOPRIGHT = "Top Right",
	LEFT = "Left",
	CENTER = "Center",
	RIGHT = "Right",
	BOTTOMLEFT = "Bottom Left",
	BOTTOM = "Bottom",
	BOTTOMRIGHT = "Bottom Right",
}

function M.FormatHeadingText(text, isDisabled, indentLevel)
	return M.FormatText(text, isDisabled, indentLevel or 0, true)
end

function M.FormatLabelText(text, isDisabled, indentLevel)
	return M.FormatText(text, isDisabled, indentLevel or 1, false)
end

function M.FormatText(text, isDisabled, indentLevel, isHeading)
	local color = isDisabled and DISABLED_FONT_COLOR or (isHeading and NORMAL_FONT_COLOR or HIGHLIGHT_FONT_COLOR)
	local prefix = string.rep("  ", indentLevel or 0)
	return color:WrapTextInColorCode(prefix .. text)
end

function M.GenerateOrderedArgs()
	local currentOrder = 0

	return setmetatable({}, {
		__newindex = function (t, k, v)
			currentOrder = currentOrder + 1
			v.order = currentOrder
			rawset(t, k, v)
		end,
	})
end

function M.GetConfigAttribute(info, ...)
	local name = info.arg.name
	local config = info.arg.config or A:GetModuleConfig(name)
	local key, keyR, keyG, keyB, keyA = info.arg.key, info.arg.keyR or "r", info.arg.keyG or "g", info.arg.keyB or "b", info.arg.keyA or "a"

	if info.type == "color" then
		return config[keyR], config[keyG], config[keyB], config[keyA]
	elseif info.type == "multiselect" then
		return config[key][...]
	else
		return config[key]
	end
end

function M.SetConfigAttribute(info, ...)
	local name = info.arg.name
	local config = info.arg.config or A:GetModuleConfig(name)
	local key, keyR, keyG, keyB, keyA = info.arg.key, info.arg.keyR or "r", info.arg.keyG or "g", info.arg.keyB or "b", info.arg.keyA or "a"

	if info.type == "color" then
		config[keyR], config[keyG], config[keyB], config[keyA] = ...
	elseif info.type == "multiselect" then
		config[key][...] = select(2, ...)
	else
		config[key] = ...
	end

	M:SendMessage("SIXUI_CONFIG_UPDATE", name, config, key, ...)
end

function M.GetGeneralAttribute(info, ...)
	local name = info[#info]
	local c, g, k = string.split(".", name)

	if info.type == "color" then
		local c = D.db.profile[c][g][k]
		return c.r, c.g, c.b, c.a
	elseif info.type == "multiselect" then
		return D.db.profile[c][g][k][...]
	else
		return D.db.profile[c][g][k]
	end
end

function M.SetGeneralAttribute(info, ...)
	local name = info[#info]
	local c, g, k = string.split(".", name)

	if info.type == "color" then
		local c = D.db.profile[c][g][k]
		c.r, c.g, c.b, c.a = ...
	elseif info.type == "multiselect" then
		D.db.profile[c][g][k][...] = select(2, ...)
	else
		D.db.profile[c][g][k] = ...
	end

	M:SendMessage("SIXUI_CONFIG_UPDATE", name, ...)
end

function M.GetModuleEnabled(info)
	return A:IsModuleEnabled(info.arg.name)
end

function M.SetModuleEnabled(info, state)
	A:SetModuleState(info.arg.name, state)
end

-------------------------------------------------------------------------------
-- Options Table Generators
-------------------------------------------------------------------------------

function M.GenerateActionBarsOptionsTable()
	local BlizzardActionBar = A:GetModule("Blizzard ActionBar")
	local optionsTable = {
		type = "group",
		name = "Action Bars",
		args = M.GenerateOrderedArgs(),
	}

	for _, barConfig, label in BlizzardActionBar:IterateBarConfigs() do
		optionsTable.args[label] = {
			name = M.FormatHeadingText(label),
			type = "group",
			get = M.GetConfigAttribute,
			set = M.SetConfigAttribute,
			args = {
				actionBarLayout = {
					name = "Action Bar Layout",
					type = "group",
					inline = true,
					args = {
						reverseRowsOnHorizontalBars = {
							name = "Reverse Rows on Horizontal Bars",
							desc = "Adjust the row order on horizontal bars to grow down instead of up. By default, Blizzard sets the row order to grow up.",
							type = "toggle",
							width = "full",
							arg = {name = "Blizzard ActionBar", config = barConfig, key = "reverseRowsOnHorizontalBars"},
						},
					},
				},
				actionBarVisibility = {
					name = "Action Bar Visbility",
					type = "group",
					inline = true,
					args = {
						stateText = {
							name = "State Text",
							desc = "Control the action bar's visibility with custom state text.",
							type = "input",
							multiline = true,
							width = "full",
							arg = {name = "Blizzard ActionBar", config = barConfig, key = "stateText"},
						},
					},
				},
				actionButtons = {
					name = "Action Buttons",
					type = "group",
					inline = true,
					args = {
						showBindingText = {
							name = "Show Binding Text",
							desc = "Show binding text on your action buttons.",
							type = "toggle",
							width = "full",
							arg = {name = "Blizzard ActionBar", config = barConfig, key = "showBindingText"},
						},
						showMacroText = {
							name = "Show Macro Text",
							desc = "Show macro text on your action buttons.",
							type = "toggle",
							width = "full",
							arg = {name = "Blizzard ActionBar", config = barConfig, key = "showMacroText"},
						},
						showTextWithClutterKey = {
							name = "Show Text with Clutter Key",
							desc = "Show binding text and macro text on your action buttons while the clutter key is down.",
							type = "toggle",
							width = "full",
							arg = {name = "Blizzard ActionBar", config = barConfig, key = "showTextWithClutterKey"},
						},
					},
				},
			},
		}
	end

	return optionsTable
end

function M.GenerateDisplaysOptionsTable()
	local DisplayGroup = A:GetModule("Display Group")
	local groupOptions = DisplayGroup:GetGroupOptions()
	local groupSorting = DisplayGroup:GetGroupSorting()
	local optionsTable = {
		name = "Displays",
		type = "group",
		args = M.GenerateOrderedArgs(),
	}

	for _, groupName in next, groupSorting do
		optionsTable.args[groupName] = {
			name = M.FormatHeadingText(groupOptions[groupName]),
			type = "group",
			disabled = true,
			args = {},
		}

		for displayName, displayConfig, dataObject in DisplayGroup:IterateDisplayConfigs(groupName) do
			optionsTable.args[displayName] = {
				name = M.FormatLabelText(displayName, not displayConfig.enabled),
				type = "group",
				args = {
					description = {
						order = 0,
						name = format("%s\n\n", dataObject.description or ""),
						type = "description",
						hidden = not dataObject.description,
						fontSize = "medium",
					},
					display = {
						order = 1,
						name = "Display",
						type = "group",
						inline = true,
						args = {
							enabled = {
								order = 1,
								name = "Enabled",
								type = "toggle",
								get = function () return displayConfig.enabled end,
								set = function (_, enabled)
									displayConfig.enabled = enabled
									DisplayGroup[enabled and "CreateDisplay" or "DeleteDisplay"](DisplayGroup, displayName, displayConfig)
									DisplayGroup:UpdateDisplayGroup(displayConfig.group)
								end,
							},
							group = {
								order = 2,
								name = "Group",
								type = "select",
								style = "dropdown",
								values = function () return groupOptions end,
								sorting = function () return groupSorting end,
								get = function () return displayConfig.group end,
								set = function (_, group)
									displayConfig.group = group
									DisplayGroup:UpdateDisplay(displayName, displayConfig)
									DisplayGroup:UpdateDisplayGroups()
								end,
							},
							order = {
								order = 3,
								name = "Order",
								type = "range",
								min = 0,
								max = 10,
								step = 1,
								get = function () return displayConfig.order end,
								set = function (_, order)
									displayConfig.order = order
									DisplayGroup:UpdateDisplay(displayName, displayConfig)
									DisplayGroup:UpdateDisplayGroup(displayConfig.group)
								end,
							},
							width = {
								order = 4,
								name = "Width",
								type = "range",
								min = 0,
								max = 1000,
								step = 1,
								get = function () return displayConfig.width end,
								set = function (_, width)
									displayConfig.width = width
									DisplayGroup:UpdateDisplay(displayName, displayConfig)
									DisplayGroup:UpdateDisplayGroup(displayConfig.group)
								end,
							},
						},
					},
					attributes = {
						order = 2,
						name = "Attributes",
						type = "multiselect",
						hidden = dataObject.type == "progress bar",
						values = function () return DisplayGroup:GetAttributeOptions() end,
						get = function (_, key) return displayConfig[key] end,
						set = function (_, key, state)
							displayConfig[key] = state
							DisplayGroup:UpdateDisplay(displayName, displayConfig)
						end,
					},
					color = {
						order = 3,
						name = "Colors",
						type = "group",
						inline = true,
						hidden = not (dataObject.type == "progress bar" or displayConfig.glyph or displayConfig.label or displayConfig.text or displayConfig.value or displayConfig.suffix),
						args = {
							useCustomBarColor = {
								order = 1,
								name = "Use Custom Bar Color",
								type = "toggle",
								hidden = dataObject.type ~= "progress bar",
								get = function () return displayConfig.useCustomBarColor end,
								set = function (_, state)
									displayConfig.useCustomBarColor = state
									DisplayGroup:UpdateDisplay(displayName, displayConfig)
								end,
							},
							barColor = {
								order = 2,
								name = "Bar Color",
								type = "color",
								hidden = dataObject.type ~= "progress bar",
								disabled = not displayConfig.useCustomBarColor,
								get = function () return displayConfig.barR, displayConfig.barG, displayConfig.barB end,
								set = function (_, r, g, b)
									displayConfig.barR, displayConfig.barG, displayConfig.barB = r, g, b
									DisplayGroup:UpdateDisplay(displayName, displayConfig)
								end,
							},
							glyphColor = {
								order = 4,
								name = "Glyph Color",
								type = "color",
								hidden = not displayConfig.glyph,
								get = function () return displayConfig.glyphR, displayConfig.glyphG, displayConfig.glyphB end,
								set = function (_, r, g, b)
									displayConfig.glyphR, displayConfig.glyphG, displayConfig.glyphB = r, g, b
									DisplayGroup:UpdateDisplay(displayName, displayConfig)
								end,
							},
							labelColor = {
								order = 6,
								name = "Label Color",
								type = "color",
								hidden = not displayConfig.label,
								get = function () return displayConfig.labelR, displayConfig.labelG, displayConfig.labelB end,
								set = function (_, r, g, b)
									displayConfig.labelR, displayConfig.labelG, displayConfig.labelB = r, g, b
									DisplayGroup:UpdateDisplay(displayName, displayConfig)
								end,
							},
							textColor = {
								order = 6,
								name = "Text Color",
								type = "color",
								hidden = not displayConfig.text,
								get = function () return displayConfig.textR, displayConfig.textG, displayConfig.textB end,
								set = function (_, r, g, b)
									displayConfig.textR, displayConfig.textG, displayConfig.textB = r, g, b
									DisplayGroup:UpdateDisplay(displayName, displayConfig)
								end,
							},
							valueColor = {
								order = 8,
								name = "Value Color",
								type = "color",
								hidden = not displayConfig.value,
								get = function () return displayConfig.valueR, displayConfig.valueG, displayConfig.valueB end,
								set = function (_, r, g, b)
									displayConfig.valueR, displayConfig.valueG, displayConfig.valueB = r, g, b
									DisplayGroup:UpdateDisplay(displayName, displayConfig)
								end,
							},
							suffixColor = {
								order = 10,
								name = "Suffix Color",
								type = "color",
								hidden = not displayConfig.suffix,
								get = function () return displayConfig.suffixR, displayConfig.suffixG, displayConfig.suffixB end,
								set = function (_, r, g, b)
									displayConfig.suffixR, displayConfig.suffixG, displayConfig.suffixB = r, g, b
									DisplayGroup:UpdateDisplay(displayName, displayConfig)
								end,
							},
						},
					},
					icon = {
						order = 4,
						name = "Icon",
						type = "group",
						inline = true,
						hidden = function () return not displayConfig.icon end,
						args = {
							iconSize = {
								order = 1,
								name = "Size",
								type = "range",
								min = 1,
								max = 30,
								step = 1,
								get = function () return displayConfig.iconSize end,
								set = function (_, iconSize)
									displayConfig.iconSize = iconSize
									DisplayGroup:UpdateDisplay(displayName, displayConfig)
								end,
							},
							iconZoom = {
								order = 2,
								name = "Zoom",
								type = "range",
								min = 0,
								max = 100,
								step = 1,
								get = function () return displayConfig.iconZoom end,
								set = function (_, iconZoom)
									displayConfig.iconZoom = iconZoom
									DisplayGroup:UpdateDisplay(displayName, displayConfig)
								end,
							},
						},
					},
					glyph = {
						order = 5,
						name = "Glyph",
						type = "group",
						inline = true,
						hidden = function () return not displayConfig.glyph end,
						args = {
							glyphOverride = {
								order = 3,
								name = "Override",
								type = "select",
								style = "dropdown",
								values = function () return DisplayGroup:GetGlyphOptions() end,
								get = function () return displayConfig.glyphOverride end,
								set = function (_, glyphOverride)
									displayConfig.glyphOverride = glyphOverride
									DisplayGroup:UpdateDisplay(displayName, displayConfig)
								end,
							},
						},
					},
				},
			}
		end
	end

	return optionsTable
end

function M:GenerateGeneralOptionsTable()
	local CoreLayout = A:GetModule("Core Layout")
	local CoreSetup = A:GetModule("Core Setup")
	local LibSharedMedia = LibStub:GetLibrary("LibSharedMedia-3.0")

	return {
		name = "General",
		type = "group",
		get = M.GetGeneralAttribute,
		set = M.SetGeneralAttribute,
		args = {
			border = {
				name = "Border",
				type = "group",
				args = {
					background = {
						order = 1,
						name = "Background",
						type = "group",
						inline = true,
						args = {
							["border.background.color"] = {
								name = "Background Color",
								type = "color",
								hasAlpha = true,
							},
							["border.background.inset"] = {
								name = "Inset",
								type = "range",
								min = 0,
								max = 10,
								step = 1,
							},
						},
					},
					stroke = {
						order = 2,
						name = "Stroke",
						type = "group",
						inline = true,
						args = {
							["border.stroke.color"] = {
								order = 1,
								name = "Stroke Color",
								type = "color",
								hasAlpha = true,
							},
							["border.stroke.anchorOffset"] = {
								order = 2,
								name = "Inset",
								type = "range",
								min = 0,
								max = 10,
								step = 1,
							},
							["border.stroke.width"] = {
								order = 3,
								name = "Width",
								type = "range",
								min = 1,
								max = 10,
								step = 1,
							},
						},
					},
					accent = {
						order = 3,
						name = "Accent",
						type = "group",
						inline = true,
						args = {
							["border.accent.color"] = {
								order = 1,
								name = "Accent Color",
								type = "color",
								hasAlpha = true,
							},
							["border.accent.anchorOffset"] = {
								order = 2,
								name = "Inset",
								type = "range",
								min = 0,
								max = 10,
								step = 1,
							},
							["border.accent.width"] = {
								order = 3,
								name = "Width",
								type = "range",
								min = 1,
								max = 10,
								step = 1,
							},
						},
					},
				},
			},
			clutter = {
				name = "Clutter",
				type = "group",
				args = {
					modifierKey = {
						name = "Modifier Key",
						type = "select",
						style = "radio",
						values = C.Enum.ModifierKeyLabel,
						get = M.GetConfigAttribute,
						set = M.SetConfigAttribute,
						arg = {name = "Miscellaneous Clutter", key = "modifierKey"},
					},
				},
			},
			color = {
				name = "Colors",
				type = "group",
				args = { 
					debuff = {
						name = "Debuffs",
						type = "group",
						inline = true,
						args = {
							["color.debuff.curse"] = {
								name = "Curse",
								type = "color",
								hasAlpha = true,
							},
							["color.debuff.disease"] = {
								name = "Disease",
								type = "color",
								hasAlpha = true,
							},
							["color.debuff.magic"] = {
								name = "Magic",
								type = "color",
								hasAlpha = true,
							},
							["color.debuff.other"] = {
								name = "Other",
								type = "color",
								hasAlpha = true,
							},
							["color.debuff.poison"] = {
								name = "Poison",
								type = "color",
								hasAlpha = true,
							},
						},
					},
					faction = {
						name = "Factions",
						type = "group",
						inline = true,
						args = {
							["color.faction.alliance"] = {
								name = "Alliance",
								type = "color",
								hasAlpha = true,
							},
							["color.faction.horde"] = {
								name = "Horde",
								type = "color",
								hasAlpha = true,
							},
						},
					},
					rating = {
						name = "Ratings",
						type = "group",
						inline = true,
						args = {
							["color.rating.gold"] = {
								order = 1,
								name = "Gold",
								type = "color",
								hasAlpha = true,
							},
							["color.rating.silver"] = {
								order = 2,
								name = "Silver",
								type = "color",
								hasAlpha = true,
							},
							["color.rating.bronze"] = {
								order = 3,
								name = "Bronze",
								type = "color",
								hasAlpha = true,
							},
							["color.rating.untimed"] = {
								order = 4,
								name = "Untimed",
								type = "color",
								hasAlpha = true,
							},
						},
					},
					reaction = {
						name = "Reactions",
						type = "group",
						inline = true,
						args = {
							["color.reaction.hostile"] = {
								name = "Hostile",
								type = "color",
								hasAlpha = true,
							},
							["color.reaction.neutral"] = {
								name = "Neutral",
								type = "color",
								hasAlpha = true,
							},
							["color.reaction.friendly"] = {
								name = "Friendly",
								type = "color",
								hasAlpha = true,
							},
						},
					},
					reputation = {
						name = "Reputations",
						type = "group",
						inline = true,
						args = {
							["color.reputation.exceptionallyHostile"] = {
								order = 1,
								name = "Exceptionally Hostile",
								type = "color",
								hasAlpha = true,
							},
							["color.reputation.veryHostile"] = {
								order = 2,
								name = "Very Hostile",
								type = "color",
								hasAlpha = true,
							},
							["color.reputation.hostile"] = {
								order = 3,
								name = "Hostile",
								type = "color",
								hasAlpha = true,
							},
							["color.reputation.neutral"] = {
								order = 4,
								name = "Neutral",
								type = "color",
								hasAlpha = true,
							},
							["color.reputation.friendly"] = {
								order = 5,
								name = "Friendly",
								type = "color",
								hasAlpha = true,
							},
							["color.reputation.veryFriendly"] = {
								order = 6,
								name = "Very Friendly",
								type = "color",
								hasAlpha = true,
							},
							["color.reputation.exceptionallyFriendly"] = {
								order = 7,
								name = "Exceptionally Friendly",
								type = "color",
								hasAlpha = true,
							},
							["color.reputation.exalted"] = {
								order = 8,
								name = "Exalted",
								type = "color",
								hasAlpha = true,
							},
						},
					},
					resource = {
						name = "Resources",
						type = "group",
						inline = true,
						args = {
							["color.resource.honor"] = {
								name = "Honor",
								type = "color",
								hasAlpha = true,
							},
							["color.resource.renown"] = {
								name = "Renown",
								type = "color",
								hasAlpha = true,
							},
							["color.resource.restedXP"] = {
								name = "Rested XP",
								type = "color",
								hasAlpha = true,
							},
							["color.resource.valor"] = {
								name = "Valor",
								type = "color",
								hasAlpha = true,
							},
							["color.resource.xp"] = {
								name = "XP",
								type = "color",
								hasAlpha = true,
							},
						},
					},
					utility = {
						name = "Utility",
						type = "group",
						inline = true,
						args = {
							["color.utility.fallback"] = {
								name = "Fallback",
								type = "color",
								hasAlpha = true,
							},
							["color.utility.guild"] = {
								name = "Guild",
								type = "color",
								hasAlpha = true,
							},
							["color.utility.highlight"] = {
								name = "Highlight",
								type = "color",
								hasAlpha = true,
							},
						},
					},
				},
			},
			font = {
				name = "Font",
				type = "group",
				args = {
					appearance = {
						order = 1,
						name = "Appearance",
						type = "group",
						inline = true,
						args = {
							["font.appearance.name"] = {
								name = "Font",
								type = "select",
								style = "dropdown",
								dialogControl = "LSM30_Font",
								values = LibSharedMedia:HashTable(LibSharedMedia.MediaType.FONT),
							},
						},
					},
					override = {
						order = 2,
						name = "Override",
						type = "group",
						inline = true,
						args = {
							["font.appearance.overrideGameFonts"] = {
								name = "Game Fonts",
								desc = "Override game fonts with SixUI's font.",
								type = "toggle",
								width = "full",
							},
							["font.appearance.overrideSharedMediaFonts"] = {
								name = "Shared Media Fonts (Requires Reload)",
								desc = "Override shared media fonts with SixUI's font. Font changes require a UI reload.",
								type = "toggle",
								width = "full",
							},
						},
					},
					textFontSize = {
						order = 3,
						name = "Text Font Size",
						type = "group",
						inline = true,
						args = {
							["font.textFontSize.medium"] = {
								name = "Chat",
								desc = "Chat text font size. Default: 13.",
								type = "range",
								min = 1,
								max = 20,
								step = 1,
							},
							["font.textFontSize.large"] = {
								name = "Heading",
								desc = "Heading text font size. Default: 14.",
								type = "range",
								min = 1,
								max = 20,
								step = 1,
							},
							["font.textFontSize.regular"] = {
								name = "Regular",
								desc = "Regular text font size. Default: 12.",
								type = "range",
								min = 1,
								max = 20,
								step = 1,
							},
						},
					},
					glyphFontSize = {
						order = 4,
						name = "Glyph Font Size",
						type = "group",
						inline = true,
						args = {
							["font.glyphFontSize.regular"] = {
								name = "Display",
								desc = "Display glyph font size. Default: 12.",
								type = "range",
								min = 1,
								max = 20,
								step = 1,
							},
							["font.glyphFontSize.large"] = {
								name = "Minimap",
								desc = "Minimap glyph font size. Default: 16.",
								type = "range",
								min = 1,
								max = 20,
								step = 1,
							},
							["font.glyphFontSize.small"] = {
								name = "Tooltip",
								desc = "Tooltip glyph font size. Default: 7.",
								type = "range",
								min = 1,
								max = 20,
								step = 1,
							},
						},
					},
				},
			},
			layout = {
				name = "Layout",
				type = "group",
				args = {
					description = {
						order = 0,
						name = "Changing the layout in SixUI sets the active profile of other AddOns to 'SixUI [layout name]' if it exists, or 'SixUI' if not. SixUI's Action Bars and Raid Frames options are specific to each layout.",
						type = "description",
						fontSize = "medium",
					},
					activeLayout = {
						name = "Active Layout",
						type = "select",
						style = "radio",
						values = function () return CoreLayout:GetOptions() end,
						get = function () return CoreLayout:GetLayout() end,
						set = function (_, layout) CoreLayout:SetLayout(layout) end,
					},
				},
			},
			reset = {
				type = "group",
				name = "Reset",
				args = {
					resetAddOn = {
						name = "Reset AddOn",
						type = "group",
						inline = true,
						args = {
							resetAddOnDescription = {
								order = 1,
								name = "Reset the SixUI profile for a specific AddOn.",
								type = "description",
							},
							resetAddOnDropdown = {
								name = "Reset AddOn",
								type = "select",
								style = "dropdown",
								values = CoreSetup:ListAddOns(),
								get = function () end,
								set = function (_, target) StaticPopup_Show("SIXUI_RESET_ADDON", target, nil, CoreSetup:FindVariablesByAddOn(target)) end,
								width = "full",
							},
						},
					},
					resetAll = {
						name = "Reset All",
						type = "group",
						inline = true,
						args = {
							resetAllDescription = {
								order = 1,
								name = "Reset the SixUI profile for all AddOns.",
								type = "description",
							},
							resetAllButton = {
								name = "Reset All",
								type = "execute",
								func = function () StaticPopup_Show("SIXUI_RESET_ALL") end,
								width = "full",
							},
						},
					},
					loadCVars = {
						name = "Load CVars",
						type = "group",
						inline = true,
						args = {
							loadCVarsDescription = {
								order = 1,
								name = "Load SixUI's recommended CVars.",
								type = "description",
							},
							loadCVarsButton = {
								name = "Load CVars",
								type = "execute",
								func = function () StaticPopup_Show("SIXUI_RESET_CVARS") end,
								width = "full",
							},
						},
					},
					changeProfile = {
						name = "Change Profile",
						type = "group",
						inline = true,
						args = {
							changeProfileDescription = {
								order = 1,
								name = "Change the active profile in all supported AddOns to SixUI's profile.",
								type = "description",
							},
							changeProfileButton = {
								name = "Change Profile",
								type = "execute",
								confirm = function () return format("The active profile will be changed to '%s' in all supported AddOns.", CoreLayout:GetProfile()) end,
								func = function () CoreSetup:SetProfile() end,
								width = "full",
							},
						},
					},
				},
			},
		},
	}
end

function M.GenerateMiscellaneousOptionsTable()
	local AutomationInvite = A:GetModule("Automation Invite")
	local AutomationSell = A:GetModule("Automation Sell")
	local optionsTable = {
		name = "Miscellaneous",
		type = "group",
		get = M.GetConfigAttribute,
		set = M.SetConfigAttribute,
		args = M.GenerateOrderedArgs(),
	}

	optionsTable.args.Automations = {
		type = "group",
		name = M.FormatHeadingText("Automations"),
		disabled = true,
		args = {},
	}

	optionsTable.args["Automation Busy"] = {
		type = "group",
		name = M.FormatLabelText("Busy", A:IsModuleDisabled("Automation Busy")),
		args = {
			description = {
				order = 0,
				name = "Auto-accepts ready checks and quests while busy or holding the Shift key.\n\n",
				type = "description",
				fontSize = "medium",
			},
			automation = {
				name = "Automation",
				type = "group",
				inline = true,
				args = {
					enabled = {
						name = "Enabled",
						type = "toggle",
						get = M.GetModuleEnabled,
						set = M.SetModuleEnabled,
						arg = {name = "Automation Busy"},
					},
				},
			},
		},
	}

	optionsTable.args["Automation Emote"] = {
		type = "group",
		name = M.FormatLabelText("Emote", A:IsModuleDisabled("Automation Emote")),
		args = {
			description = {
				order = 0,
				name = "Auto-thanks the player who resurrected you.\n\n",
				type = "description",
				fontSize = "medium",
			},
			automation = {
				name = "Automation",
				type = "group",
				inline = true,
				args = {
					enabled = {
						name = "Enabled",
						type = "toggle",
						get = M.GetModuleEnabled,
						set = M.SetModuleEnabled,
						arg = {name = "Automation Emote"},
					},
				},
			},
		},
	}

	optionsTable.args["Automation Invite"] = {
		type = "group",
		name = M.FormatLabelText("Invite", A:IsModuleDisabled("Automation Invite")),
		args = {
			description = {
				order = 0,
				name = "Auto-accepts group invitations from players you know.\n\n",
				type = "description",
				fontSize = "medium",
			},
			automation = {
				order = 0,
				name = "Automation",
				type = "group",
				inline = true,
				args = {
					enabled = {
						name = "Enabled",
						type = "toggle",
						get = M.GetModuleEnabled,
						set = M.SetModuleEnabled,
						arg = {name = "Automation Invite"},
					},
				},
			},
			allow = {
				name = "Allow",
				type = "group",
				inline = true,
				args = {
					allowFriends = {
						name = "Friends",
						desc = "Auto-accept group invitations from friends.",
						type = "toggle",
						arg = {name = "Automation Invite", key = "allowFriends"},
					},
					allowClubMembers = {
						name = "Guild Members",
						desc = "Auto-accept group invitations from guild members.",
						type = "toggle",
						arg = {name = "Automation Invite", key = "allowClubMembers"},
					},
					allowLastCharacterWhispered = {
						name = "Last Character Whispered",
						desc = "Auto-accept group invitations from the last character you whispered.",
						type = "toggle",
						arg = {name = "Automation Invite", key = "allowLastCharacterWhispered"},
					},
				},
			},
			disable = {
				name = "Disable",
				type = "group",
				inline = true,
				args = {
					disableInQueue = {
						name = "In a Queue",
						desc = "Disable auto-accept while in a queue.",
						type = "toggle",
						arg = {name = "Automation Invite", key = "disableInQueue"},
					},
					disableInInstance = {
						name = "In an Instance",
						desc = "Disable auto-accept while in an instance.",
						type = "toggle",
						arg = {name = "Automation Invite", key = "disableInInstance"},
					},
				},
			},
			blacklist = {
				name = "Ignore List",
				type = "group",
				inline = true,
				args = {
					new = {
						order = 1,
						name = "New",
						desc = "Add a BattleTag, guild rank, or character name to the auto-accept ignore list.",
						type = "input",
						get = function () end,
						set = function (_, target) AutomationInvite:Ignore(target) end,
					},
					delete = {
						order = 2,
						name = "Delete",
						desc = "Delete an entry from the auto-accept ignore list.",
						type = "select",
						style = "dropdown",
						values = function () return AutomationInvite:GetIgnoreList() end,
						get = function () end,
						set = function (_, target) AutomationInvite:Unignore(target) end,
					},
				},
			},
		},
	}

	optionsTable.args["Automation Repair"] = {
		type = "group",
		name = M.FormatLabelText("Repair", A:IsModuleDisabled("Automation Repair")),
		args = {
			description = {
				order = 0,
				name = "Auto-repairs items when you visit a vendor.\n\n",
				type = "description",
				fontSize = "medium",
			},
			automation = {
				name = "Automation",
				type = "group",
				inline = true,
				args = {
					enabled = {
						name = "Enabled",
						type = "toggle",
						get = M.GetModuleEnabled,
						set = M.SetModuleEnabled,
						arg = {name = "Automation Repair"},
					},
				},
			},
			fundSource = {
				name = "Fund Source",
				type = "group",
				inline = true,
				args = {
					useCharacterGold = {
						name = "Character Gold",
						desc = "Fund auto-repair using character gold.",
						type = "toggle",
						arg = {name = "Automation Repair", key = "useCharacterGold"},
					},
					useGuildGold = {
						name = "Guild Gold",
						desc = "Fund auto-repair using guild gold.",
						type = "toggle",
						arg = {name = "Automation Repair", key = "useGuildGold"},
					},
				},
			},
		},
	}

	optionsTable.args["Automation Sell"] = {
		type = "group",
		name = M.FormatLabelText("Sell", A:IsModuleDisabled("Automation Sell")),
		args = {
			description = {
				order = 0,
				name = "Auto-sells junk when you visit a vendor.\n\n",
				type = "description",
				fontSize = "medium",
			},
			automation = {
				name = "Automation",
				type = "group",
				inline = true,
				args = {
					enabled = {
						name = "Enabled",
						type = "toggle",
						get = M.GetModuleEnabled,
						set = M.SetModuleEnabled,
						arg = {name = "Automation Sell"},
					},
				},
			},
			blacklist = {
				name = "Ignore List",
				type = "group",
				inline = true,
				args = {
					new = {
						order = 1,
						name = "New",
						desc = "Add an item link to the auto-sell ignore list.",
						type = "input",
						get = function () end,
						set = function (_, target) AutomationSell:Ignore(target) end,
					},
					delete = {
						order = 2,
						name = "Delete",
						desc = "Delete an item link from the auto-sell ignore list.",
						type = "select",
						style = "dropdown",
						values = function () return AutomationSell:GetIgnoreList() end,
						get = function () end,
						set = function (_, target) AutomationSell:Unignore(target) end,
					},
				},
			},
		},
	}

	optionsTable.args["Automation Watch"] = {
		type = "group",
		name = M.FormatLabelText("Watch", A:IsModuleDisabled("Automation Watch")),
		args = {
			description = {
				order = 0,
				name = "Auto-watches the faction you last gained reputation with.\n\n",
				type = "description",
				fontSize = "medium",
			},
			automation = {
				name = "Automation",
				type = "group",
				inline = true,
				args = {
					enabled = {
						name = "Enabled",
						type = "toggle",
						get = M.GetModuleEnabled,
						set = M.SetModuleEnabled,
						arg = {name = "Automation Watch"},
					},
				},
			},
		},
	}

	optionsTable.args.Blizzard = {
		type = "group",
		name = M.FormatHeadingText("Blizzard"),
		disabled = true,
		args = {},
	}

	optionsTable.args["Buffs Bars"] = {
		type = "group",
		name = M.FormatLabelText("Buff Frame"),
		args = {
			appearance = {
				name = "Appearance",
				type = "group",
				inline = true,
				args = {
					enabled = {
						name = "Show Buff Bars",
						desc = "Show buff bars instead of duration text. Hold the clutter key to reveal duration text as needed.",
						type = "toggle",
						width = "full",
						get = M.GetModuleEnabled,
						set = M.SetModuleEnabled,
						arg = {name = "Buff Bars"},
					},
					showCollapseAndExpandButton = {
						name = "Show Collapse Buffs Button",
						desc = "Show the Collapse Buffs button. When hidden, hold the clutter key to reveal as needed.",
						type = "toggle",
						width = "full",
						get = M.GetConfigAttribute,
						set = M.SetConfigAttribute,
						arg = {name = "Blizzard Miscellaneous", key = "showCollapseAndExpandButton"},
					},
				},
			},
		},
	}

	optionsTable.args["Chat Frame"] = {
		name = M.FormatLabelText("Chat Frame"),
		type = "group",
		get = M.GetModuleEnabled,
		set = M.SetModuleEnabled,
		args = {
			chatDisplay = {
				name = "Chat Display",
				type = "group",
				inline = true,
				args = {
					abbreviations = {
						name = "Abbreviate Channel Names",
						desc = "Shorten chat channel names for a cleaner chat interface and easier reading.",
						type = "toggle",
						width = "full",
						arg = {name = "Chat Abbreviations"},
					},
					link = {
						name = "Convert URLs and Faction Names to Clickable Links",
						desc = "Transform URLs and faction names in chat messages into clickable links for easy access.",
						type = "toggle",
						width = "full",
						arg = {name = "Chat Link"},
					},
					scrollIndicator = {
						name = "Display Scroll Indicator Button",
						desc = "Add a scroll indicator button to the chat frame for quick navigation to new messages.",
						type = "toggle",
						width = "full",
						arg = {name = "Chat ScrollIndicator"},
					},
				},
			},
			chatInteraction = {
				name = "Chat Interaction",
				type = "group",
				inline = true,
				args = {
					copy = {
						name = "Enable Chat Message Copy",
						desc = "Right-click on the chat frame to copy its text, making it easy to share or save information",
						type = "toggle",
						width = "full",
						arg = {name = "Chat Copy"},
					},
					cycle = {
						name = "Enable Channel Cycling",
						desc = "Press the Tab key while typing a message to quickly cycle through available chat channels.",
						type = "toggle",
						width = "full",
						arg = {name = "Chat Cycle"},
					},
					invite = {
						name = "Alt-click to Invite Players",
						desc = "Hold the Alt key and click on a player's name in chat to send them a group or raid invite.",
						type = "toggle",
						width = "full",
						arg = {name = "Chat Invite"},
					},
				},
			},
			chatFiltersAndOptions = {
				name = "Chat Filters and Options",
				type = "group",
				inline = true,
				args = {
					filters = {
						name = "Filter Messages with Foreign Characters",
						desc = "Automatically hide chat messages containing non-English characters for a cleaner chat experience.",
						type = "toggle",
						width = "full",
						arg = {name = "Chat Filters"},
					},
					history = {
						name = "Remember Chat History between Sessions",
						desc = "Preserve your chat history when you log out, allowing you to continue conversations when you return.",
						type = "toggle",
						width = "full",
						arg = {name = "Chat History"},
					},
				},
			},
			chatNotifications = {
				name = "Chat Notifications",
				type = "group",
				inline = true,
				args = {
					sounds = {
						name = "Play Sounds for New Whispers and Guild Messages",
						desc = "Play an audible notification when you receive a new whisper or guild message, ensuring you don't miss important communications.",
						type = "toggle",
						width = "full",
						arg = {name = "Chat Sounds"},
					},
				},
			},
		},
	}

	optionsTable.args["Blizzard ObjectiveTracker"] = {
		type = "group",
		name = M.FormatLabelText("Objective Tracker"),
		args = {
			appearance = {
				name = "Appearance",
				type = "group",
				inline = true,
				args = {
					showArtwork = {
						name = "Show Artwork",
						desc = "Show the Objective Tracker's artwork.",
						type = "toggle",
						width = "full",
						arg = {name = "Blizzard ObjectiveTracker", key = "showArtwork"},
					},
					showButtons = {
						name = "Show Buttons",
						desc = "Show the Objective Tracker's buttons. When hidden, hold the clutter key to reveal as needed.",
						type = "toggle",
						width = "full",
						arg = {name = "Blizzard ObjectiveTracker", key = "showButtons"},
					},
				},
			},
			visibility = {
				name = "Visbility",
				type = "group",
				inline = true,
				args = {
					stateText = {
						name = "State Text",
						desc = "Control the Objective Tracker's visibility with custom state text.",
						type = "input",
						multiline = true,
						width = "full",
						arg = {name = "Blizzard ObjectiveTracker", key = "stateText"},
					},
				},
			},
		},
	}

	optionsTable.args.Integrations = {
		type = "group",
		name = M.FormatHeadingText("Integrations"),
		disabled = true,
		args = {},
	}

	optionsTable.args["Integration AdiBags"] = {
		type = "group",
		name = M.FormatLabelText("AdiBags", A:IsModuleDisabled("Integration AdiBags")),
		hidden = not IsAddOnLoaded("AdiBags"),
		args = {
			description = {
				order = 0,
				name = "Skins inventory slots.\n\n",
				type = "description",
				fontSize = "medium",
			},
			integration = {
				name = "Integration",
				type = "group",
				inline = true,
				args = {
					enabled = {
						name = "Enabled",
						type = "toggle",
						get = M.GetModuleEnabled,
						set = M.SetModuleEnabled,
						arg = {name = "Integration AdiBags"},
					},
				},
			},
		},
	}

	optionsTable.args["Integration Bagnon"] = {
		type = "group",
		name = M.FormatLabelText("Bagnon", A:IsModuleDisabled("Integration Bagnon")),
		hidden = not IsAddOnLoaded("Bagnon"),
		args = {
			description = {
				order = 0,
				name = "Skins inventory slots.\n\n",
				type = "description",
				fontSize = "medium",
			},
			integration = {
				name = "Integration",
				type = "group",
				inline = true,
				args = {
					enabled = {
						name = "Enabled",
						type = "toggle",
						get = M.GetModuleEnabled,
						set = M.SetModuleEnabled,
						arg = {name = "Integration Bagnon"},
					},
				},
			},
		},
	}

	optionsTable.args["Integration Dominos"] = {
		type = "group",
		name = M.FormatLabelText("Dominos", A:IsModuleDisabled("Integration Dominos")),
		hidden = not IsAddOnLoaded("Dominos"),
		args = {
			description = {
				order = 0,
				name = "Skins action buttons.\n\n",
				type = "description",
				fontSize = "medium",
			},
			integration = {
				order = 0,
				name = "Integration",
				type = "group",
				inline = true,
				args = {
					enabled = {
						name = "Enabled",
						type = "toggle",
						get = M.GetModuleEnabled,
						set = M.SetModuleEnabled,
						arg = {name = "Integration Dominos"},
					},
				},
			},
			clutterKey = {
				name = "Clutter Key",
				type = "group",
				inline = true,
				args = {
					showBindingText = {
						name = "Show Binding Text",
						type = "toggle",
						width = "full",
						arg = {name = "Integration Dominos", key = "showBindingText"},
					},
					showMacroText = {
						name = "Show Macro Text",
						type = "toggle",
						width = "full",
						arg = {name = "Integration Dominos", key = "showMacroText"},
					},
				},
			},
		},
	}

	optionsTable.args["Integration Gnosis"] = {
		type = "group",
		name = M.FormatLabelText("Gnosis", A:IsModuleDisabled("Integration Gnosis")),
		hidden = not IsAddOnLoaded("Gnosis"),
		args = {
			description = {
				order = 0,
				name = "Skins cast bars. Colors cast bars by class and reaction.\n\n",
				type = "description",
				fontSize = "medium",
			},
			integration = {
				name = "Integration",
				type = "group",
				inline = true,
				args = {
					enabled = {
						name = "Enabled",
						type = "toggle",
						get = M.GetModuleEnabled,
						set = M.SetModuleEnabled,
						arg = {name = "Integration Gnosis"},
					},
				},
			},
		},
	}

	optionsTable.args["Integration Grid2"] = {
		type = "group",
		name = M.FormatLabelText("Grid2", A:IsModuleDisabled("Integration Grid2")),
		hidden = not IsAddOnLoaded("Grid2"),
		args = {
			description = {
				order = 0,
				name = "Skins raid frames. Adjusts raid frames using SixUI's Raid Frames options.\n\n",
				type = "description",
				fontSize = "medium",
			},
			integration = {
				name = "Integration",
				type = "group",
				inline = true,
				args = {
					enabled = {
						name = "Enabled",
						type = "toggle",
						get = M.GetModuleEnabled,
						set = M.SetModuleEnabled,
						arg = {name = "Integration Grid2"},
					},
				},
			},
		},
	}

	optionsTable.args["Integration ShadowedUnitFrames"] = {
		type = "group",
		name = M.FormatLabelText("Shadowed Unit Frames", A:IsModuleDisabled("Integration ShadowedUnitFrames")),
		hidden = not IsAddOnLoaded("ShadowedUnitFrames"),
		args = {
			description = {
				order = 0,
				name = "Skins unit frames and auras.\n\n",
				type = "description",
				fontSize = "medium",
			},
			integration = {
				name = "Integration",
				type = "group",
				inline = true,
				args = {
					enabled = {
						name = "Enabled",
						type = "toggle",
						get = M.GetModuleEnabled,
						set = M.SetModuleEnabled,
						arg = {name = "Integration ShadowedUnitFrames"},
					},
				},
			},
		},
	}

	optionsTable.args["Integration Skada"] = {
		type = "group",
		name = M.FormatLabelText("Skada", A:IsModuleDisabled("Integration Skada")),
		hidden = not IsAddOnLoaded("Skada"),
		args = {
			description = {
				order = 0,
				name = "Displays more detailed columns while the clutter key is down.\n\n",
				type = "description",
				fontSize = "medium",
			},
			integration = {
				name = "Integration",
				type = "group",
				inline = true,
				args = {
					enabled = {
						name = "Enabled",
						type = "toggle",
						get = M.GetModuleEnabled,
						set = M.SetModuleEnabled,
						arg = {name = "Integration Skada"},
					},
				},
			},
		},
	}

	optionsTable.args["Integration VuhDo"] = {
		type = "group",
		name = M.FormatLabelText("VuhDo", A:IsModuleDisabled("Integration VuhDo")),
		hidden = not IsAddOnLoaded("VuhDo"),
		args = {
			description = {
				order = 0,
				name = "Skins raid frames. Adjusts raid frames using SixUI's Raid Frames options.\n\n",
				type = "description",
				fontSize = "medium",
			},
			integration = {
				name = "Integration",
				type = "group",
				inline = true,
				args = {
					enabled = {
						name = "Enabled",
						type = "toggle",
						get = M.GetModuleEnabled,
						set = M.SetModuleEnabled,
						arg = {name = "Integration VuhDo"},
					},
				},
			},
			roleOverrides = {
				name = "Role Overrides",
				type = "group",
				inline = true,
				args = {
					overrideHunterRole = {
						name = "Classify Hunters as Ranged",
						type = "toggle",
						width = "full",
						arg = {name = "Integration VuhDo", key = "overrideHunterRole"},
					},
					overrideHolyPaladinRole = {
						name = "Classify Holy Paladins as Melee",
						type = "toggle",
						width = "full",
						arg = {name = "Integration VuhDo", key = "overrideHolyPaladinRole"},
					},
				},
			},
		},
	}

	optionsTable.args["Integration WeakAuras"] = {
		type = "group",
		name = M.FormatLabelText("WeakAuras", A:IsModuleDisabled("Integration WeakAuras")),
		hidden = not IsAddOnLoaded("WeakAuras"),
		args = {
			description = {
				order = 0,
				name = "Skins icons and progress bars.\n\n",
				type = "description",
				fontSize = "medium",
			},
			integration = {
				name = "Integration",
				type = "group",
				inline = true,
				args = {
					enabled = {
						name = "Enabled",
						type = "toggle",
						get = M.GetModuleEnabled,
						set = M.SetModuleEnabled,
						arg = {name = "Integration WeakAuras"},
					},
				},
			},
		},
	}

	optionsTable.args.Tootips = {
		type = "group",
		name = M.FormatHeadingText("Tooltips"),
		disabled = true,
		args = {},
	}

	optionsTable.args["Display Friends"] = {
		type = "group",
		name = M.FormatLabelText("Friends"),
		args = {
			categories = {
				name = "Categories",
				type = "group",
				inline = true,
				args = {
					categorizeByNote = {
						name = "Custom",
						desc = "Create custom categories from your friend's notes. To do this, type a category name inside parentheses in their note, for example, '(Keystones)' or 'Ben (Keystones)'.",
						type = "toggle",
						width = "full",
						arg = {name = "Display Friends", key = "categorizeByNote"},
					},
					categorizeFavorites = {
						name = "Favorites",
						desc = "Create a category for your favorite friends.",
						type = "toggle",
						width = "full",
						arg = {name = "Display Friends", key = "categorizeFavorites"},
					},
					categorizeCharacterFriends = {
						name = "World of Warcraft",
						desc = "Create a category for your World of Warcraft friends.",
						type = "toggle",
						width = "full",
						arg = {name = "Display Friends", key = "categorizeCharacterFriends"},
					},
				},
			},
			columns = {
				name = "Columns",
				type = "multiselect",
				values = C.Enum.FriendColumn,
				arg = {name = "Display Friends", key = "columns"},
			},
			friends = {
				name = "Friends",
				type = "group",
				inline = true,
				args = {
					showMobile = {
						name = "Show Mobile",
						type = "toggle",
						width = "full",
						arg = {name = "Display Friends", key = "showMobile"},
					},
					showOffline = {
						name = "Show Offline",
						type = "toggle",
						width = "full",
						arg = {name = "Display Friends", key = "showOffline"},
					},
				},
			},
		},
	}

	optionsTable.args["Display Guild"] = {
		type = "group",
		name = M.FormatLabelText("Guild"),
		args = {
			columns = {
				name = "Columns",
				type = "multiselect",
				values = C.Enum.GuildColumn,
				arg = {name = "Display Guild", key = "columns"},
			},
		},
	}

	return optionsTable
end

function M.GenerateRaidFramesOptionsTable()
	local optionsTable = {
		name = "Raid Frames",
		type = "group",
		get = M.GetConfigAttribute,
		set = M.SetConfigAttribute,
		args = M.GenerateOrderedArgs(),
	}

	for groupSize, raidFrameConfig, label in A:IterateRaidFrameConfigs() do
		optionsTable.args[label] = {
			name = M.FormatHeadingText(label),
			type = "group",
			order = groupSize,
			args = {
				buttonSize = {
					order = 1,
					name = "Button Size",
					type = "group",
					inline = true,
					args = {
						width = {
							order = 1,
							name = "Width",
							type = "range",
							min = 1,
							max = 250,
							step = 1,
							arg = {name = "Core RaidFrameConfig", config = raidFrameConfig, key = "width"},
						},
						height = {
							order = 2,
							name = "Height",
							type = "range",
							min = 1,
							max = 150,
							step = 1,
							arg = {name = "Core RaidFrameConfig", config = raidFrameConfig, key = "height"},
						},
					},
				},
				panelPosition = {
					order = 2,
					name = "Panel Position",
					type = "group",
					inline = true,
					args = {
						point = {
							order = 1,
							name = "Panel Anchor Point",
							type = "select",
							values = M.AnchorPointValues,
							arg = {name = "Core RaidFrameConfig", config = raidFrameConfig, key = "point"},
						},
						relativeTo = {
							order = 2,
							name = "Anchor Frame",
							type = "input",
							arg = {name = "Core RaidFrameConfig", config = raidFrameConfig, key = "relativeTo"},
						},
						relativePoint = {
							order = 3,
							name = "Anchor Frame Anchor Point",
							type = "select",
							values = M.AnchorPointValues,
							arg = {name = "Core RaidFrameConfig", config = raidFrameConfig, key = "relativePoint"},
						},
						offsetX = {
							order = 4,
							name = "X Offset",
							type = "range",
							min = -2880,
							max = 2880,
							step = 0.1,
							arg = {name = "Core RaidFrameConfig", config = raidFrameConfig, key = "offsetX"},
						},
						offsetY = {
							order = 5,
							name = "Y Offset",
							type = "range",
							min = -2880,
							max = 2880,
							step = 0.1,
							arg = {name = "Core RaidFrameConfig", config = raidFrameConfig, key = "offsetY"},
						},
					},
				},
			},
		}
	end

	return optionsTable
end

-------------------------------------------------------------------------------
-- Options Tables
-------------------------------------------------------------------------------

M.optionsTable = {
	name = "SixUI",
	type = "group",
	args = {
		welcomeDescription = {
			order = 1,
			type = "description",
			name = format("Welcome to SixUI version %s.", C.VERSION),
		},
	},
}
